;;     Game - PP The Space Mercenary
;;     Copyright (C) 2019  TeamBotella
;; 
;;     This program is free software: you can redistribute it and/or modify
;;     it under the terms of the GNU General Public License as published by
;;     the Free Software Foundation, either version 3 of the License, or
;;     (at your option) any later version.
;; 
;;     This program is distributed in the hope that it will be useful,
;;     but WITHOUT ANY WARRANTY; without even the implied warranty of
;;     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;     GNU General Public License for more details.
;; 
;;     You should have received a copy of the GNU General Public License
;;     along with this program.  If not, see <https://www.gnu.org/licenses/>.
.include "entity_manager.h.s"
.include "cpctelera.h.s"
.include "assets/assets.h.s"
.include "src/sys/phys.h.s"
.include "src/sys/collision.h.s"
.include "src/sys/render.h.s"
max_entities = 25

_num_entities:: .db 0
_last_elem_ptr:: .dw _entity_array
_entity_array::
    .ds max_entities*entity_size

_dirToRoom: .db #0x00
_pos_on_door: ;; Y, X -> Por Little Endian
    .dw #0x5405         ;; Right    0
    .dw #0x5440         ;; Left     1
    .dw #0x9023         ;; Up       2
    .dw #0x1023         ;; Down     3

entityman_init::
    xor a
    ld (_num_entities), a
    ld hl, #_entity_array
    ld (_last_elem_ptr), hl
ret

entityman_getEntityArray::
    ld hl, #_entity_array
    ret

entityman_getNumEntities::
    ld a, (_num_entities)
    ret

entityman_create::
    ld de, (#_last_elem_ptr)
    ld bc, #entity_size
    ldir

    ld a, (#_num_entities)
    inc a
    ld (#_num_entities), a

    ex de, hl
    ld (#_last_elem_ptr), hl

    ret

;; DESTROYS: AF, HL, DE, BC
entityman_eraseAllExceptPlayer::

    call rendersys_erase_all_entities

    ld hl, #_entity_array+entity_size*2
    ld (#_last_elem_ptr), hl

    ld a, #2
    ld (#_num_entities), a


    ret

;; Decrease health of target from attack of dealer
;; INPUT:       IX: Pointer of attacker
;;              IY: Target pointer
;; DESTROYS:     A:
entity_damage::

    ld a, e_team(ix)
    cp e_team(iy)
    ret z

    ld a, e_stun(ix)
    or a
    ret nz

    ld a, e_health(ix)
    dec a
    ld e_health(ix), a

    jr nz, _next

    ld a, #STATUS_DEAD
    ld e_status(ix), a
    ld a, e_last_x(ix)
    ld e_x(ix), a
    ld a, e_last_y(ix)
    ld e_y(ix), a

    call erase_previous_instance_ix_entity
    ld a, e_type(ix)
    cp #TYPE_BULLET
    jr z, _next

    ld a, e_dsprite_l(ix)
    ld e_sprite_l(ix), a
    ld a, e_dsprite_h(ix)
    ld e_sprite_h(ix), a
    call rend_entity

    _next:
    ld a, #TEAM_ENEMY
    cp e_team(iy)
    jr nz, _no_enemigo

    ld a, #e_ai_st_attack
    ld e_ai_status(iy), a

    _no_enemigo:
    ld a, e_health(iy)
    dec a
    ld e_health(iy), a

    ret nz

    ld a, #STATUS_DEAD
    ld e_status(iy), a
    ld a,e_last_x(iy)
    ld e_x(iy), a
    ld a, e_last_y(iy)
    ld e_y(iy), a
    call erase_previous_instance_iy_entity
    ld a, e_dsprite_l(iy)
    ld e_sprite_l(iy), a
    ld a, e_dsprite_h(iy)
    ld e_sprite_h(iy), a

    ret

checkLife::
    ld hl, #_entity_array + e_status
    ld a, (hl)

    cp #STATUS_DEAD
    ret

entityman_player_by_door::

    ld a, (_dirToRoom)
    add a, a

    ld de, #_pos_on_door
    ld l, a
    ld h, #0
    add hl, de
    ld d, (hl)
    inc hl
    ld e, (hl)

    ld hl, #_entity_array+e_x
    ld (hl), d

    ld hl, #_entity_array+e_y
    ld (hl), e

    ret

;; Return (X,Y) of player character and where is it facing
;; DESTROYS: HL
;; OUTPUT:  D = Coordinate X
;;          E = Coordinate Y
;;          A = Facing
entityman_getPlayerPos::
    ld hl, #_entity_array+e_x
    ld d, (hl)

    ld hl, #_entity_array+e_y
    ld e, (hl)

    ld hl, #_entity_array+e_facing
    ld a, (hl)

    ret

;; DATA FOR BULLETS
positionOnShoot::
    ;; Shooted to east
    .db #5 , #8                 ;; Position      X,  Y
    .db #2 , #0                 ;; Velocities   VX, VY
    .db #2, #2                  ;; WIDHT, HEIGHT
    .dw _sp_horizontalbullet_0  ;; Sprite Used
    ;; Shooted to west
    .db #-3, #8                 ;; Position      X,  Y
    .db #-2, #0                 ;; Velocities   VX, VY
    .db #2, #2                  ;; WIDHT, HEIGHT
    .dw _sp_horizontalbullet_1  ;; Sprite Used
    ;; Shooted to north
    .db #2 , #-10                ;; Position      X,  Y
    .db #0 , #-4                ;; Velocities   VX, VY
    .db #1, #8                  ;; WIDHT, HEIGHT
    .dw _sp_verticalbullet_0    ;; Sprite Used
    ;; Shooted to south
    .db #2 , #20                ;; Position      X,  Y
    .db #0 , #4                 ;; Velocities   VX, VY
    .db #1, #8                  ;; WIDHT, HEIGHT
    .dw _sp_verticalbullet_1    ;; Sprite Used

bullet_shooted::

    ld hl, #_entity_array+entity_size

    ex de, hl

    ld__ixl_e
    ld__ixh_d

    call entityman_getPlayerPos

    ld e_facing(ix), a

    ld hl, #positionOnShoot
    rlc a
    rlc a
    rlc a
    call sumar_puntero

    ;; IX = Bullet
    ;; Setting positions
    ld a, (hl)
    add d
    call check_illegal_x_left
    ret c
    call check_illegal_x_right
    ret nc
    ld e_x(ix), a
    ld e_last_x(ix), a

    inc hl
    ld a, (hl)
    add e
    call check_illegal_y_up
    ret c
    call check_illegal_y_down
    ret nc
    ld e_y(ix), a
    ld e_last_y(ix), a

    ld a, #STATUS_ACTIVE
    ld e_status(ix), a

    ld a, #1
    ld e_health(ix), a

    inc hl
    ld a, (hl)
    ld e_vx(ix), a

    inc hl
    ld a, (hl)
    ld e_vy(ix), a

    inc hl
    ld a, (hl)
    ld e_w(ix), a

    inc hl
    ld a, (hl)
    ld e_h(ix), a

    inc hl
    ld a, (hl)
    ld e_sprite_l(ix), a
    ld e_dsprite_l(ix), a

    inc hl
    ld a, (hl)
    ld e_sprite_h(ix), a
    ld e_dsprite_h(ix), a

    call rend_entity

    ret

;; Checks if there is any bullet in game
;; DESTROYS: HL, A, B
;; OUTPUT: Flag Z, REGISTER: F
no_bullet::
    ld hl, #_entity_array+entity_size+e_status

    ld a, (hl)
    cp #STATUS_DEAD

    ret

;; Kills BULLET on IX
;; INPUT: IX = Bullet
;; DESTROYS: AF
deactive_ix_bullet::

    ld a, #STATUS_DEAD
    cp e_status(ix)
    ret z
    ld e_status(ix), a

    ld a, e_last_x(ix)
    ld e_x(ix), a
    ld a, e_last_y(ix)
    ld e_y(ix), a

    call erase_previous_instance_ix_entity

    ret

;; Kills BULLET on IY
;; INPUT: IY = Bullet
;; DESTROYS: AF
deactive_iy_bullet::

    ld a, #STATUS_DEAD
    ld e_status(iy), a

    ld a, e_last_x(iy)
    ld e_x(iy), a
    ld a, e_last_y(iy)
    ld e_y(iy), a

    call erase_previous_instance_iy_entity

ret
;; Substracts 1 byte from 2 bytes
;; INPUT:  HL = Memory Adress to be substracted
;;          A = Number to sub
;; OUTPUT: HL = Memory Adress after substract
restar_a_puntero::
    sub_hl_a
    ret

changeRoomDir::
    ld (_dirToRoom), a
    ret

;; Set knockback to entities that collisionates
;; INPUT: IX = Entity 1
;;        IY = Entity 2
;; OUTPUT: F = NC if entity has no consecuences
;; DESTROYS: A
;; DATA FOR PLAYER KNOCKBACK
velocityOnContact::
    ;; Knockback to east
    .db #6 ,     #0          ;; Velocities   VX, VY
    ;; Knockback to west
    .db #-6,    #0           ;; Velocities   VX, VY
    ;; Knockback to north
    .db #0 ,     #-8         ;; Velocities   VX, VY
    ;; Knockback to south
    .db #0 ,     #8          ;; Velocities   VX, VY
entity_knockback::
    ld a, e_team(ix)
    cp e_team(iy)
    ret z

    ld a, e_type(ix)
    cp #TYPE_BULLET
    jr z, bullet_knockback

    ld a, e_facing(iy)
    add a
    ld hl, #velocityOnContact
    call sumar_puntero

    ld a, (hl)
    ld e_vx(ix), a
    inc hl
    ld a, (hl)
    ld e_vy(ix), a

    call check_update_pos

    push iy
    ld hl, #_num_entities
    ld a, (hl)
    dec a
    call collision_set_entity_number
    call check_collision_entity
    pop iy

    ld a, #40
    ld e_stun(ix), a
    ld e_stun(iy), a

    ret

    bullet_knockback:
        xor a
        ld e_vx(iy), a
        ld e_vy(iy), a

        ld a, #5
        ld e_stun(iy), a

    ret

check_consecuenced_entity_ix::
    ld a, e_type(ix)
    cp #value_of_consecuence

    ret

check_consecuenced_entity_iy::
    ld a, e_type(iy)
    cp #value_of_consecuence

    ret

sumar_puntero::
    add_hl_a
    ret