;;     Game - PP The Space Mercenary
;;     Copyright (C) 2019  TeamBotella
;; 
;;     This program is free software: you can redistribute it and/or modify
;;     it under the terms of the GNU General Public License as published by
;;     the Free Software Foundation, either version 3 of the License, or
;;     (at your option) any later version.
;; 
;;     This program is distributed in the hope that it will be useful,
;;     but WITHOUT ANY WARRANTY; without even the implied warranty of
;;     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;     GNU General Public License for more details.
;; 
;;     You should have received a copy of the GNU General Public License
;;     along with this program.  If not, see <https://www.gnu.org/licenses/>.
.include "man/entity_manager.h.s"
.include "sys/phys.h.s"
.include "sys/input.h.s"
.include "sys/render.h.s"
.include "sys/collision.h.s"
.include "man/room.h.s"
.include "assets/assets.h.s"
.include "sys/ai_control.h.s"
.include "man/patrol.h.s"
.include "man/animation.h.s"
.include "sys/animation.h.s"
.globl cpct_setDrawCharM0_asm
.globl cpct_drawStringM0_asm
.globl cpct_getScreenPtr_asm
.globl cpct_drawSolidBox_asm
.globl cpct_isAnyKeyPressed_asm
.globl cpct_scanKeyboard_f_asm
.globl cpct_zx7b_decrunch_s_asm
;; TEMPORAL
.include "cpctelera.h.s"

DefineEntity player, 36, 80, 0, 0, 4, 16, #0xC000, #_sp_pp_06, #-1, 5, 0, #FACING_SOUTH, #TYPE_PLAYER, #STATUS_ACTIVE, e_ai_st_noAI, #0, _pp_walking_down, #10, #TEAM_PLAYER, _sp_deadpp
DefineEntity bullet, 0, 0, 0, 0, 2,  2, #0xC000, #0x1000, #-1, 1, 0, #FACING_EAST, #TYPE_BULLET, #STATUS_DEAD, e_ai_st_noAI, #0, #0x0000, #0, #TEAM_PLAYER, #0x0000

string: .asciz "Play:  Press '1'"
stringCO: .asciz "Controls:  Press '2'"
stringME: .asciz "Menu:  Press '3'"
stringGO: .asciz "Game Over"
;;"Press R to restart"

game_init::
    call patrol_init
    call entityman_init
    call entityman_getEntityArray
    call sys_ai_control_init
    call sys_input_init
    call sys_physics_init

    call rooms_init

    call rendersys_init

    ld hl, #player
    call entityman_create

    ld hl, #bullet
    call entityman_create

    xor a
    call load_room

    call entityman_getEntityArray
    call collision_init

    ret

game_update::
    
    call checkLife
    call z, game_over

    call sys_input_update
    call nz, character_shooting

    call entityman_getNumEntities
    call sys_ai_control_update
    call entityman_getNumEntities
    call sys_physics_update
    call collision_update
    call room_update

    call bullet_timeout_next_itr

    ret

game_render::

    call entityman_getNumEntities
    call rendersys_update

    ret

game_menu::
    ld de, #0xFFFF
    ld hl, #_menu_z_end
    call cpct_zx7b_decrunch_s_asm

    presskey:
        ld   l, #11
        ld   h, #3
        call cpct_setDrawCharM0_asm
        ld   hl, #0xE328
        ld   iy, #string 
        call cpct_drawStringM0_asm
        ld   hl, #0xE3C0
        ld   iy, #stringCO
        call cpct_drawStringM0_asm
        call cpct_scanKeyboard_f_asm
        ld hl, #Key_1
        call cpct_isKeyPressed_asm
        ret nz
        ld hl, #Key_2
        call cpct_isKeyPressed_asm
        call nz, showControls
    jr z, presskey

    ret

showControls:
    ld de, #0xFFFF
    ld hl, #_controls_z_end
    call cpct_zx7b_decrunch_s_asm

    ld    l, #11
    ld    h, #3
    call cpct_setDrawCharM0_asm
    ld   hl, #0xE468
    ld   iy, #string
    call cpct_drawStringM0_asm
    ld   hl, #0xE508
    ld   iy, #stringME
    call cpct_drawStringM0_asm

    presskey1:
        call cpct_scanKeyboard_f_asm
        ld hl, #Key_1
        call cpct_isKeyPressed_asm
        ret nz
        ld hl, #Key_3
        call cpct_isKeyPressed_asm
        call nz, game_menu
    jr z, presskey1

    ret

game_over::
    ld    l, #9
    ld    h, #5
    call cpct_setDrawCharM0_asm
    ld   hl, #0xE246
    ld   iy, #stringGO
    call cpct_drawStringM0_asm

    _waitingRestart:
        call cpct_scanKeyboard_f_asm
        ld hl, #Key_R
        call cpct_isKeyPressed_asm
        jr z, _waitingRestart

    ret

bullet_timeout_constant = 20
bulletTimeout: .db #0x00
bullet_timeout_next_itr:
    ld a, (#bulletTimeout)
    or a
    ret z

    dec a
    ld (#bulletTimeout), a 

    ret

character_shooting::

    ld a, (#bulletTimeout)
    or a
    ret nz

    call no_bullet
    ret nz

    call bullet_shooted
    ld a, #bullet_timeout_constant
    ld (#bulletTimeout), a
    ret