;;     Game - PP The Space Mercenary
;;     Copyright (C) 2019  TeamBotella
;; 
;;     This program is free software: you can redistribute it and/or modify
;;     it under the terms of the GNU General Public License as published by
;;     the Free Software Foundation, either version 3 of the License, or
;;     (at your option) any later version.
;; 
;;     This program is distributed in the hope that it will be useful,
;;     but WITHOUT ANY WARRANTY; without even the implied warranty of
;;     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;     GNU General Public License for more details.
;; 
;;     You should have received a copy of the GNU General Public License
;;     along with this program.  If not, see <https://www.gnu.org/licenses/>.
.include "man/entity_manager.h.s"
.include "man/room.h.s"
.include "cpctelera.h.s"
.include "src/sys/phys.h.s"
.include "src/sys/render.h.s"
.globl cpct_setDrawCharM0_asm
.globl cpct_drawStringM0_asm
.globl cpct_isKeyPressed_asm
.globl cpct_scanKeyboard_f_asm
.globl menu

stringWIN: .asciz "You escaped!!"

collision_init::
    ld (_ent_ptr), hl

    ret

collision_set_entity_number::
    ld (_inner_loop_counter), a
    ret

check_collision_entity::

    push ix
    pop iy
    _inner_loop:
        ld de, #entity_size
        add iy, de

        _is_Alive:
            ld a, e_status(iy)
            cp #STATUS_DEAD
            jr z, _next_entity

        call check_collision
        call nc, _colisionado

        _next_entity:
            _inner_loop_counter= .+1
            ld a, #0x00
            dec a
            ret z
            ld (_inner_loop_counter), a

    jr _inner_loop

    ret

;; Colisiones en 2D ->
;;         X o X+W esta entre X2 y  X2+W
;;         Y o Y+W esta entre Y2 y  Y2+W
;; Detecta la colisión con otra entidad
;; DESTRUYE : IY, AF, BC, DE
;;
collision_update::
_ent_ptr = .+2
    ld ix, #0x0000
    call entityman_getNumEntities
    ld (_outer_loop_counter),a

_outer_loop:
    _outer_loop_counter = .+1
    ld a, #0x00
    dec a
    ret z

    ld (_outer_loop_counter), a
    ld (_inner_loop_counter), a

    ld a, e_type(ix)
    cp #TYPE_ENEMY
    ret z

    ld a, e_status(ix)
    cp #STATUS_DEAD
    jr z, _next

    call check_collision_entity

    _next:
        ld de, #entity_size
        add ix, de
        jr _outer_loop

check_collision::

    call _x_compare
    ret c
    call _y_compare

    ret

_x_compare::
    ld a, e_x(ix)
    add e_w(ix)
    sub #1
    cp e_x(iy)
    ret c

    ld a, e_x(iy)
    add e_w(iy)
    sub #1
    cp e_x(ix)

    ret

_y_compare::
    ld a, e_y(ix)
    add e_h(ix)
    sub #1
    cp e_y(iy)
    ret c

    ld a, e_y(iy)
    add e_h(iy)
    sub #1
    cp e_y(ix)

    ret

_colisionado::
    ld a, #TYPE_DOOR
    cp e_type(iy)
    jr z, _puerta
    ld a, #TYPE_OBJECT
    cp e_type(iy)
    jr z, _objeto
    ld a, #TYPE_SHIP
    cp e_type(iy)
    jr z, _victory
    _entidad:
        call check_consecuenced_entity_ix
        jr nc, _just_collision
        call check_consecuenced_entity_iy
        jr nc, _just_collision

        _is_Stunned:
        ld a, e_stun(ix)
        or a
        call z, entity_damage

        call entity_knockback

        ret

        _just_collision:
            ld a, #TYPE_BULLET
            cp e_type(ix)
            jr z, _bullet_impact

            ld a, #TYPE_ENEMY
            cp e_type(iy)
            ret z

            ld a, e_last_x(ix)
            ld e_x(ix), a

            ld a, e_last_y(ix)
            ld e_y(ix), a

            ret

            _bullet_impact:
                call deactive_ix_bullet
            ret
    _puerta:
        ld a, #TYPE_PLAYER
        cp e_type(ix)
        ret nz

        ld a, e_doorId(iy)
        call change_room

        ld a, e_facing(iy)
        call changeRoomDir

        ret
    _objeto:
        ld a, #TYPE_PLAYER
        cp e_type(ix)
        ret nz

        ld a, e_ai_y(ix)
        inc a
        ld e_ai_y(ix), a

        ld e_status(iy), #STATUS_DEAD

        call erase_previous_instance_iy_entity

        ret
    _victory:
        ld    l, #11
        ld    h, #5
        call cpct_setDrawCharM0_asm
        ld   hl, #0xE242
        ld   iy, #stringWIN
        call cpct_drawStringM0_asm

        _waitingRestart:
            call cpct_scanKeyboard_f_asm
            ld hl, #Key_R
            call cpct_isKeyPressed_asm
            jr z, _waitingRestart

        jp menu