;;     Game - PP The Space Mercenary
;;     Copyright (C) 2019  TeamBotella
;; 
;;     This program is free software: you can redistribute it and/or modify
;;     it under the terms of the GNU General Public License as published by
;;     the Free Software Foundation, either version 3 of the License, or
;;     (at your option) any later version.
;; 
;;     This program is distributed in the hope that it will be useful,
;;     but WITHOUT ANY WARRANTY; without even the implied warranty of
;;     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;     GNU General Public License for more details.
;; 
;;     You should have received a copy of the GNU General Public License
;;     along with this program.  If not, see <https://www.gnu.org/licenses/>.
.include "phys.h.s"
.include "man/entity_manager.h.s"
.include "man/room.h.s"

;;
;; INITS the physics system
;;
sys_physics_init::
    ld (_ent_ptr), hl
    ret

;; Updates all physics components of entities
;; INPUT: A = Number of elements in the array
;; Destroys: AF, BC, DE, IX, HL
sys_physics_update:: 
    ld (_counter), a     ;; B = Number of Entities

_ent_ptr = .+2
    ld ix, #0x0000
    _update_loop:
        ld a, e_type(ix)
        cp #TYPE_OBSTACLE
        ret z

        ld a, e_status(ix)
        cp #STATUS_DEAD
        jr z, _next_entity

        ld a, e_stun(ix)
        or a
        jr z, etiq
        dec a
        ld e_stun(ix), a
        jr _next_entity

        etiq:
        ld a, e_type(ix)
        cp #TYPE_ENEMY
        jr nz, calculo

        ld a, e_doorId(ix)
        xor #-1
        ld e_doorId(ix), a
        jr z, _next_entity

        calculo:
        call check_update_pos

        _next_entity:
            _counter = .+1
            ld a, #0x00
            dec a
            ret z

            ld (_counter), a
            ld de, #entity_size
            add ix, de
            jr _update_loop

;; Checks if its new position is possible, in that case updates the position using velocity
;; INPUT: IX = Entity to be moved
;; DESTROYS: AF, C, HL
check_update_pos::
    ;; UPDATE X
        ld a, #screen_width + 1
        sub e_w(ix)
        ld c, a

        ld a, e_x(ix)
        ld e_last_x(ix), a
        add e_vx(ix)
        cp c
        jr nc, invalid_x
        cp #lower_left_limit
        jr c, invalid_x
        valid_x:
            ld e_x(ix), a
            jr endif_x
        invalid_x:
            ld a, e_type(ix)
            cp #TYPE_BULLET
            call z, deactive_ix_bullet
            xor a
            ld e_vx(ix), a
        endif_x:
        ;; UPDATE Y
        ld a, #screen_height + 1
        sub e_h(ix)
        ld c, a

        ld a, e_y(ix)
        ld e_last_y(ix), a
        add e_vy(ix)
        cp c
        jr nc, invalid_y
        cp #lower_up_limit
        jr c, invalid_y
        valid_y:
            ld e_y(ix), a
            jr endif_y
        invalid_y:
            ld a, e_type(ix)
            cp #TYPE_BULLET
            call z, deactive_ix_bullet
            xor a
            ld e_vy(ix), a
        endif_y:
    ret

;; Checks if X is inside the screen
;; Output: F -> Carry flag if is outside
;; DESTROYS: C
check_illegal_x_left::
    ld c, a

    cp #lower_left_limit

    ld a, c
    ret

check_illegal_x_right::
    ld c, a

    add e_w(ix)
    cp #screen_width+1

    ld a, c
    ret

check_illegal_y_up::
    ld c, a

    cp #lower_up_limit

    ld a, c
    ret

check_illegal_y_down::
    ld c, a

    add e_h(ix)
    cp #screen_height+1

    ld a, c
    ret