;;     Game - PP The Space Mercenary
;;     Copyright (C) 2019  TeamBotella
;; 
;;     This program is free software: you can redistribute it and/or modify
;;     it under the terms of the GNU General Public License as published by
;;     the Free Software Foundation, either version 3 of the License, or
;;     (at your option) any later version.
;; 
;;     This program is distributed in the hope that it will be useful,
;;     but WITHOUT ANY WARRANTY; without even the implied warranty of
;;     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;     GNU General Public License for more details.
;; 
;;     You should have received a copy of the GNU General Public License
;;     along with this program.  If not, see <https://www.gnu.org/licenses/>.
.include "render.h.s"
.include "man/entity_manager.h.s"
.include "sys/animation.h.s"
.include "assets/assets.h.s"
.include "cpctelera.h.s"

.globl cpct_setVideoMode_asm
.globl cpct_setPalette_asm
.globl cpct_drawSolidBox_asm
.globl cpct_getScreenPtr_asm
.globl cpct_drawSprite_asm
.globl cpct_drawSpriteBlended_asm
.globl cpct_etm_setDrawTilemap4x8_ag_asm
.globl cpct_etm_drawTilemap4x8_ag_asm
.globl _pp_palette
.globl cpct_setDrawCharM0_asm
.globl cpct_drawStringM0_asm

screen_start = 0xC000

rendersys_preinit::
    ld c, #0
    call cpct_setVideoMode_asm
    ld hl, #_pp_palette
    ld de, #16
    call cpct_setPalette_asm
    cpctm_setBorder_asm HW_BLACK
    ret

rendersys_init::
    ld (_ent_ptr), hl
    ld (ent_ptr_erase), hl
    call drawTilemap
    ld hl, #rendersys_first_render_entities
    ld (_render_function_ptr), hl
    ld de, #0xC707
    ld b, #9
    ld c, #4
    ld hl, #_sp_hpart
    call cpct_drawSprite_asm
    ld de, #0xC710
    ld b, #9
    ld c, #32
    ld hl, #_sp_hudart
    call cpct_drawSprite_asm
    ret

drawTilemap::
    ld c, #20
    ld b, #25
    ld de, #20
    ld hl, #_tl_tiles_00
    call cpct_etm_setDrawTilemap4x8_ag_asm
    ld hl, #0xC000
    ld de, #_tm_map
    call cpct_etm_drawTilemap4x8_ag_asm
ret

rendersys_update::

    _render_function_ptr= .+1
    call rendersys_first_render_entities
    call rendersys_render_hud

    ret

rendersys_erase_all_entities::

    ent_ptr_erase = .+2
    ld ix, #0x0000
    call entityman_getNumEntities
    ld (_erase_ent_counter), a
    sub #2
    ret z

    _erase_loop:
        ld a, e_status(ix)
        cp #STATUS_DEAD
        call nz, erase_previous_instance_ix_entity

        _erase_ent_counter = .+1
            ld a, #0
        dec a
        ret z

        ld (_erase_ent_counter), a
        ld bc, #entity_size
        add ix, bc
        jr _erase_loop

erase_previous_instance_iy_entity::

    ;; Erases previous Instance
    ld e, e_lastVP_l(iy)
    ld d, e_lastVP_h(iy)
    ld b, e_w(iy)
    ld c, e_h(iy)
    ld l, e_lastSprite_l(iy)
    ld h, e_lastSprite_h(iy)
    call cpct_drawSpriteBlended_asm

    ret

erase_previous_instance_ix_entity::

    ;; Erases previous Instance
    ld e, e_lastVP_l(ix)
    ld d, e_lastVP_h(ix)
    ld b, e_w(ix)
    ld c, e_h(ix)
    ld l, e_lastSprite_l(ix)
    ld h, e_lastSprite_h(ix)
    call cpct_drawSpriteBlended_asm

    ret

rend_entity::
    ;; Calculate new Video Memory Pointer
    ld de, #screen_start
    ld c, e_x(ix)
    ld b, e_y(ix)
    call cpct_getScreenPtr_asm

    ;; Store Video Memory Pointer
    ld e_lastVP_l(ix), l
    ld e_lastVP_h(ix), h

    ;; Draw Entity Sprite
    ex de, hl
    ld b, e_w(ix)
    ld c, e_h(ix)
    ld l, e_sprite_l(ix)
    ld h, e_sprite_h(ix)
    ld e_lastSprite_l(ix), l
    ld e_lastSprite_h(ix), h
    call cpct_drawSpriteBlended_asm

    ret

;; INPUT:   A = Number of entities
rendersys_render_entities:
    ld (_ent_counter), a
_ent_ptr = .+2
    ld ix, #0x0000

    update_loop:

    ld a, e_type(ix)
    cp #TYPE_OBSTACLE
    ret z

    ld a, e_x(ix)
    cp e_last_x(ix)
    jr nz, _animaciones
    ld a, e_y(ix)
    cp e_last_y(ix)
    jr z, _next_entity

    _animaciones:
    ;; Spaghetti code para animaciones
    ld a, e_type(ix)
    cp #0
    jr nz, _no_pp
    ld a, e_facing(ix)
    cp e_last_facing(ix)
    jr z, _no_pp
    call change_animation_facing
    _no_pp:
    ld a, e_anim_it(ix)
    dec a
    ld e_anim_it(ix), a
    jr nz, _dib
    call check_animado
    ld a, #15
    ld e_anim_it(ix), a
    _dib:

    ;; Erases previos Instance
    call erase_previous_instance_ix_entity
    call rend_entity

    _next_entity:
    _ent_counter = .+1
        ld a, #0
        dec a
        ret z

        ld (_ent_counter), a
        ld bc, #entity_size
        add ix, bc
        jr update_loop

rendersys_first_render_entities:
    ld (_ent_counter2), a
    ld ix, (_ent_ptr)

    update_loop2:

    ld a, e_status(ix)
    cp #STATUS_DEAD
    call nz, rend_entity

    _ent_counter2 = .+1
        ld a, #0
        dec a
        jr z, change_function_ptr

        ld (_ent_counter2), a
        ld bc, #entity_size
        add ix, bc
        jr update_loop2

change_function_ptr:
    ld hl, #rendersys_render_entities
    ld (_render_function_ptr), hl
ret

change_to_first_render::
    ld hl, #rendersys_first_render_entities
    ld (_render_function_ptr), hl
ret

string0: .asciz "0/3"
string1: .asciz "1/3"
string2: .asciz "2/3"
string3: .asciz "3/3"
_finalCorazones: .dw 0xC6F0
rendersys_render_hud::
    ld a, #5
    call entityman_getEntityArray
    ld de, #e_health
    add hl, de
    ld b, (hl)
    push bc
    push af

    _bucle:
    pop af
    pop bc
    cp #0
    jr z, _endbucle
    cp b
    push bc
    push af
    jr z, _vida
    jr c, _vida

    ld hl, #_sp_hearth_1
    push hl
    jr _comun

    _vida:
    ld hl, #_sp_hearth_0
    push hl


    _comun:
    ld hl, #_finalCorazones
    ld e, (hl)
    inc hl
    ld d, (hl)
    ld b, #9
    ld c, #4
    pop hl
    call cpct_drawSprite_asm
    ld hl, #_finalCorazones
    ld e, (hl)
    ld a, e
    sub #4
    ld e, a
    ld (hl), e
    pop af
    dec a
    push af
    jr _bucle

    _endbucle:
    ld hl, #_finalCorazones
    ld (hl), #0xF0

    call entityman_getEntityArray
    ld de, #e_ai_y
    add hl, de
    ld b, (hl)

    ld a, #0
    cp b
    jr z, _0piezas
    ld a, #1
    cp b
    jr z, _1piezas
    ld a, #2
    cp b
    jr z, _2piezas
    ld a, #3
    cp b
    jr z, _3piezas

    _0piezas:
    ld hl, #string0
    jr _elegida
    _1piezas:
    ld hl, #string1
    jr _elegida
    _2piezas:
    ld hl, #string2
    jr _elegida
    _3piezas:
    ld hl, #string3
    jr _elegida

    _elegida:
    push hl

    ld    l, #15
    ld    h, #3
    call cpct_setDrawCharM0_asm
    ld de, #0xC000
    ld c, #26
    ld b, #177
    call cpct_getScreenPtr_asm
    pop iy
    call cpct_drawStringM0_asm

    ret